package controllers

import (
	"devel.mephi.ru/dyokunev/cps-api/app/common/db"
	asuModels "devel.mephi.ru/dyokunev/go-asu-models"
	"github.com/revel/revel"
	"strconv"
	"strings"
	"time"
)

type App struct {
	Controller
}

type routeInfo struct {
	Method string
	Path   string
}

func (c App) Index() revel.Result {
	if r := c.Init(); r != nil {
		return r
	}

	var routes []routeInfo
	for _, route := range revel.MainRouter.Routes {
		if len(route.ControllerNamespace) < 1 {
			continue
		}

		if route.ControllerNamespace[0:1] != strings.ToUpper(route.ControllerNamespace[0:1]) {
			continue
		}

		routes = append(routes, routeInfo{
			Path:   route.Path,
			Method: route.Method,
		})
	}

	return c.Render(routes)
}

func (c App) Login(back_url string) revel.Result {
	return c.Redirect(back_url)
}

func (c App) NoOp(service string) revel.Result {
	return c.Render(nil)
}

func (c App) Me() revel.Result {
	if r := c.Init(); r != nil {
		return r
	}

	return c.Render(c.Args["me"])
}

func (c App) ShowPermissionsShort() revel.Result {
	if r := c.Init(); r != nil {
		return r
	}

	me := c.GetMe()

	return c.Render(me.Permissions.GetShortForm())
}

func (c App) Status() revel.Result {
	if r := c.Init(); r != nil {
		return r
	}

	rows, err := db.CpsRaw.Query("SELECT MAX(dtDate) FROM events")
	if r := c.ConsiderErr(err); r != nil {
		return r
	}
	defer rows.Close()
	if !rows.Next() {
		panic("This shouldn't happend")
	}
	var dtDateMax time.Time
	err = rows.Scan(&dtDateMax)
	if err != nil {
		panic(err)
	}

	return c.Render(map[string]interface{}{
		"lastEventDatetime": dtDateMax,
	})
}

func (c App) RedirectToCalculate(unitCode string) revel.Result {
	if r := c.Init(); r != nil {
		return r
	}

	units, err := asuModels.UnitSQL.RealOnly().ActiveOnly().Select(asuModels.Unit{Code: unitCode})
	if err != nil {
		return c.RenderError(err)
	}
	unitIds := (*asuModels.Units)(&units).PrepareChildrenTree().GetRecursive().GetUnitIds()

	unitStrIds := []string{}
	for _, unitId := range unitIds {
		unitStrIds = append(unitStrIds, strconv.Itoa(unitId))
	}

	return c.Redirect("https://new.cps.mephi.ru/calculate.html?unitIds=" + strings.Join(unitStrIds, ","))
}
