package controllers

import (
	"database/sql"
	"devel.mephi.ru/dyokunev/cps-api/app/common"
	renderModels "devel.mephi.ru/dyokunev/cps-models/for_render"
	sdModels11 "devel.mephi.ru/dyokunev/go-sd-models/api11"
	helpers "devel.mephi.ru/dyokunev/go-helpers"
	"devel.mephi.ru/dyokunev/go-sdapi/sdApi4"
	"fmt"
	"github.com/revel/revel"
	"os"
	"time"
)

type People struct {
	Controller
}

func (c People) Index(person sdModels11.Person, q string, prepareFormulars, preparePasses bool, startDate, endDate time.Time) revel.Result {
	if r := c.Init(); r != nil {
		return r
	}

	scope := c.PersonSQL

	if q != "" {
		scope = scope.Where("CONCAT(fullname) LIKE ?", "%"+q+"%")
	}

	peopleRaw, err := scope.Select(person)
	if r := c.ConsiderErr(err); r != nil {
		return r
	}
	people := sdModels11.People(peopleRaw)

	if prepareFormulars {
		people.PrepareFormulars()
	}

	renderPeople := renderModels.NewPeople(people)

	if preparePasses {
		personIds := people.GetPersonIds()
		if len(personIds) > 0 {
			passesScope := c.PassSQL
			if (time.Time{}) != startDate || (time.Time{}) != endDate {
				passesScope = passesScope.SetTableQuery("`events_users_view` JOIN `activity_userids` USING (`UserID`)").Group("UserId")
				if (time.Time{}) != startDate {
					passesScope = passesScope.Where("date >= ?", startDate)
				}
				if (time.Time{}) != endDate {
					passesScope = passesScope.Where("date < ?", endDate)
				}
			}

			passes, err := passesScope.Select("person_id IN (?)", personIds)
			if err != nil && err != sql.ErrNoRows {
				c.RenderError(err)
			}

			personMap := renderPeople.ToMap()

			for _, pass := range passes {
				if pass.PersonId == nil {
					revel.ERROR.Printf("This shouldn't happened")
					continue
				}
				person := personMap[*pass.PersonId]
				if person == nil {
					revel.ERROR.Printf("This shouldn't happened")
					continue
				}
				person.Passes = append(person.Passes, pass)
			}
		}
	}

	return c.Render(renderPeople)
}

func (c People) renderNoPhoto() revel.Result {
	return c.Redirect("https://voip.mephi.ru/public/photos/no_photo.png")
}

func (c People) renderPhotoOf(personId int) revel.Result {
	filePath := fmt.Sprintf("public/photos/securityDepartment/%v.jpg", personId)

	absFilePath := "/home/site/gopath/src/devel.mephi.ru/dyokunev/cps-api/" + filePath

	if _, err := os.Stat(absFilePath); os.IsNotExist(err) {
		photo, err := sdApi4.GetPhoto(personId)
		if err != nil {
			return c.RenderError(err)
		}
		if len(photo) < 10000 {
			return c.renderNoPhoto()
		}
		err = helpers.SaveBytesToFile(absFilePath, photo)
		if err != nil {
			return c.RenderError(err)
		}
	}

	return c.Redirect("/" + filePath)
}

func (c People) GetPhoto(personId int) revel.Result {
	if r := c.Init(); r != nil {
		return r
	}

	me := c.GetMe()

	if !me.Permissions.IsFullAccess {
		allowedPeopleMap := me.Permissions.AllowedPeople.ToMap()
		if allowedPeopleMap[personId] == nil {
			return c.renderNoPhoto()
		}
	}

	return c.renderPhotoOf(personId)
}

func (c People) GetPermissions(login string) revel.Result {
	me := c.GetMe()

	if !me.Permissions.IsVoip {
		c.RenderError(fmt.Errorf("permission denied"))
	}

	if login == "" {
		c.RenderError(fmt.Errorf("\"login\" is empty"))
	}

	userInfo := common.GetUserInfo(login)
	return c.Render(userInfo.Permissions)
}

func (c People) GetShortPermissions(login string) revel.Result {
	me := c.GetMe()

	if !me.Permissions.IsVoip {
		c.RenderError(fmt.Errorf("permission denied"))
	}

	if login == "" {
		c.RenderError(fmt.Errorf("\"login\" is empty"))
	}

	userInfo := common.GetUserInfo(login)
	return c.Render(userInfo.Permissions.GetShortForm())
}
