package controllers

import (
	"database/sql"
	renderModels "devel.mephi.ru/dyokunev/cps-models/for_render"
	asuModels "devel.mephi.ru/dyokunev/go-asu-models"
	"github.com/revel/revel"
	"time"
)

type Units struct {
	Controller
}

func (c Units) scope(unit asuModels.Unit, activeOnly bool, unhiddenOnly bool, realOnly bool, notEmptyOnly bool) *asuModels.UnitScope {
	scope := c.UnitSQL.Order("Code", "DESC")

	if !unit.IsEqualsTo(asuModels.Unit{}) {
		scope = scope.Where(unit)
	}

	if activeOnly {
		scope = scope.ActiveOnly()
	}

	if unhiddenOnly {
		scope = scope.UnhiddenOnly()
	}

	if realOnly {
		scope = scope.RealOnly()
	}

	return scope
}

func (c Units) Index(unit asuModels.Unit, activeOnly, unhiddenOnly, realOnly, notEmptyOnly, setIfHasFormularsRecursive, prepareFormulars, showChildren, forceRecursiveChildren, preparePeople, ppsOnly bool, startDate, endDate time.Time) revel.Result {
	if r := c.Init(); r != nil {
		return r
	}

	me := c.GetMe()

	if preparePeople {
		prepareFormulars = true
	}

	scope := c.scope(unit, activeOnly, unhiddenOnly, realOnly, notEmptyOnly)

	units, err := scope.Select()
	if r := c.ConsiderErr(err); r != nil {
		return r
	}

	persNumbers := asuModels.Units(units).GetPersNumbers()
	unitIds := asuModels.Units(units).GetUnitIds()

	var queryArgs []interface{}
	query := "0 = 1"
	if len(persNumbers) > 0 && len(unitIds) > 0 {
		query = "PersNumber IN (?) OR OrgDiv IN (?)"
		queryArgs = []interface{}{persNumbers, unitIds}
	} else if len(unitIds) > 0 {
		query = "OrgDiv IN (?)"
		queryArgs = []interface{}{unitIds}
	}

	timesheetRowScope := c.TimesheetRowSQL.SetQueryFieldsByNames("StfCardId").SetTableQuery("`table` `table_view` JOIN formular on StaffCardId = formular.StfCardId").Group("StaffCardId")

	timesheetRowScope = timesheetRowScope.Where(query, queryArgs...)

	if (time.Time{}) != startDate {
		timesheetRowScope = timesheetRowScope.Where("ADate > ?", startDate)
	}
	if (time.Time{}) != endDate {
		timesheetRowScope = timesheetRowScope.Where("ADate < ?", endDate)
	}

	staffCardIdRows, err := timesheetRowScope.Select()
	if err == sql.ErrNoRows {
		return c.Render(renderModels.NewUnits(units))
	}
	if r := c.ConsiderErr(err); r != nil {
		return r
	}

	if prepareFormulars {
		stfCardIds := asuModels.TimesheetRows(staffCardIdRows).GetStfCardIds()
		if len(stfCardIds) > 0 {
			asuModels.Units(units).PrepareFormulars(false, ppsOnly, "StfCardId IN (?)", stfCardIds)
		} else {
			// Just to set formularsReady to true:
			asuModels.Units(units).PrepareFormulars(false, ppsOnly, "0=1")
		}
	}
	if preparePeople {
		for idx, _ := range units {
			units[idx].GetFormulars().PreparePeople()
		}
	}

	unitsForRender := renderModels.NewUnits(units)
	if unit.IsEqualsTo(asuModels.Unit{}) && len(me.Permissions.AllowedUnits) > 200 {
		unitsForRender.CalculateTree()
	} else {
		if showChildren || setIfHasFormularsRecursive {
			if forceRecursiveChildren || setIfHasFormularsRecursive {
				unitsForRender.PrepareChildrenTree()
			} else {
				unitsForRender.PrepareChildren()
			}
		}
	}

	if setIfHasFormularsRecursive {
		unitsForRender.SetIfHasFormularsRecursive()
	}

	if showChildren {
		unitsForRender.ShowChildren()
	}

	return c.Render(unitsForRender)
}
